import 'dart:convert';

import '../storage_database.dart';

class StorageListeners {
  StorageDatabase storageDatabase;

  StorageListeners(this.storageDatabase) {
    if (!storageDatabase.source.containsKey('listeners')) {
      storageDatabase.source.setData("listeners", "{}");
    }
  }

  Map get listenersData => jsonDecode(
        storageDatabase.source.getData("listeners") ?? "{}",
      );

  List<String> getPathStreamIds(String path) =>
      listenersData[path]?.keys.toList() ?? [];

  Future<bool> setListenersData(Map data) => storageDatabase.source.setData(
        "listeners",
        jsonEncode(data),
      );

  bool hasStreamId(String path, String streamId) {
    try {
      Map listenersData = this.listenersData;
      return listenersData.containsKey(path) &&
          listenersData[path].containsKey(streamId);
    } catch (e) {
      print("has stream id: $e");
      return false;
    }
  }

  initStream(String path, String streamId) {
    Map listenersData = this.listenersData;
    if (listenersData[path] == null) listenersData[path] = {};
    listenersData[path][streamId] = {"set_date": 1, "get_date": 0};
    setListenersData(listenersData);
  }

  Future<int> setDate(String path, String streamId, {int? microseconds}) async {
    Map listenersData = this.listenersData;
    int microsecondsSinceEpoch =
        microseconds ?? DateTime.now().microsecondsSinceEpoch;
    listenersData[path][streamId]["set_date"] = microsecondsSinceEpoch;
    await setListenersData(listenersData);
    return microsecondsSinceEpoch;
  }

  Future<int> getDate(String path, String streamId, {int? microseconds}) async {
    Map listenersData = this.listenersData;
    int microsecondsSinceEpoch =
        microseconds ?? DateTime.now().microsecondsSinceEpoch;
    listenersData[path][streamId]["get_date"] = microsecondsSinceEpoch;
    await setListenersData(listenersData);
    return microsecondsSinceEpoch;
  }

  Map getDates(String path, String streamId) {
    return listenersData[path][streamId];
  }
}
