import 'dart:io';

import 'package:path_provider/path_provider.dart';

import '../storage_collection.dart';
import '../storage_database.dart';
import 'explorer_directory.dart';
import 'explorer_file.dart';
import 'explorer_network_files.dart';

class StorageExplorer {
  final StorageDatabase storageDatabase;
  final Directory localIODirectory;
  late StorageCollection explorerCollection;
  late ExplorerDirectory localDirectory;

  StorageExplorer(
    this.storageDatabase,
    this.localIODirectory,
  ) {
    storageDatabase.onClear.add(initLocalDirectory);
    initLocalDirectory();
  }

  initLocalDirectory() {
    String localDirName = localIODirectory.path.split("\\").last;
    explorerCollection = StorageCollection(storageDatabase, "explorer");
    explorerCollection.document(localDirName).set({"type": "dir", "items": {}});
    localDirectory = ExplorerDirectory(
      localIODirectory,
      localDirName,
      localDirName,
      explorerCollection,
      explorerCollection.document(localDirName),
    );
  }

  static Future<StorageExplorer> getInstance(
    StorageDatabase storageDatabase, {
    String? customPath,
  }) async {
    Directory localIODirectory = Directory(
      "${(await getApplicationDocumentsDirectory()).path}\\storage_database_explorer${customPath != null ? '\\$customPath' : ''}",
    );
    if (!await localIODirectory.exists()) {
      localIODirectory = await localIODirectory.create();
    }

    return StorageExplorer(
      storageDatabase,
      localIODirectory,
    );
  }

  ExplorerNetworkFiles? networkFiles;
  initNetWorkFiles({ExplorerDirectory? cashDirictory}) async {
    networkFiles = ExplorerNetworkFiles(
      this,
      cashDirictory ?? directory('network-files'),
    );
  }

  ExplorerDirectory directory(String dirName, {bool log = true}) {
    List<String> dirNames =
        dirName.contains("/") ? dirName.split("/") : [dirName];

    Directory nIODirectory = Directory(
      "${localIODirectory.path}\\${dirNames[0]}",
    );
    if (!nIODirectory.existsSync()) nIODirectory.createSync();
    localDirectory.directoryDocument.document("items").set({
      dirNames[0]: {"type": "dir", "items": {}}
    });

    if (log) {
      for (var streamId
          in explorerCollection.storageListeners.getPathStreamIds("explorer")) {
        if (explorerCollection.storageListeners
            .hasStreamId("explorer", streamId)) {
          explorerCollection.storageListeners.getDate("explorer", streamId);
        }
      }
    }

    ExplorerDirectory explorerDirectory = ExplorerDirectory(
      nIODirectory,
      dirNames[0],
      dirNames[0],
      explorerCollection,
      localDirectory.directoryDocument.document("items").document(dirNames[0]),
    );
    for (int i = 1; i < dirNames.length; i++) {
      explorerDirectory = explorerDirectory.directory(
        dirNames[i],
      );
    }
    return explorerDirectory;
  }

  ExplorerFile file(String filename) {
    Map? dirFiles = localDirectory.directoryDocument.document("items").get();
    if (dirFiles == null || !dirFiles.containsKey(filename)) {
      localDirectory.directoryDocument.document("items").set({
        filename: {
          "type": "file",
        }
      });
    }
    File ioFile = File("${localIODirectory.path}\\$filename");
    if (!ioFile.existsSync()) {
      ioFile.createSync();
    }
    return ExplorerFile(
      ioFile,
      "",
      filename,
      explorerCollection,
      localDirectory.directoryDocument,
    );
  }

  Future clear() async => await localDirectory.clear();
}
